/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 24/07/22.
//

#include "fggl/entity/loader/loader.hpp"
#include "fggl/debug/logging.hpp"

namespace fggl::entity {

	assets::AssetRefRaw load_prototype(EntityFactory* factory, const assets::AssetGUID& guid, assets::AssetData data) {
		auto* filePath = std::get<assets::AssetPath*>(data);

		// We need to process the prototypes, and load them into the asset system.
		auto nodes = YAML::LoadAllFromFile( filePath->c_str() );
		for (const auto& node : nodes) {
			auto prefabs = node["prefabs"];

			for ( const auto& prefab : prefabs ) {
				auto name = prefab["name"].as<fggl::util::GUID>();

				#ifndef NDEBUG
					debug::info("found prefab: {}", fggl::util::guidToString(name) );
				#endif

				// set up the components
				EntitySpec entity{};
				entity.parent = prefab["parent"].as<fggl::util::GUID>(NO_PARENT);

				for (const auto& compEntry : prefab["components"]) {
					auto compId = compEntry.first.as<fggl::util::GUID>();

					ComponentSpec compSpec{};
					compSpec.config = compEntry.second;
					entity.components[compId] = compSpec;
					entity.ordering.push_back(compId);

					#ifndef NDEBUG
						debug::trace("{} has component {}", fggl::util::guidToString(name), fggl::util::guidToString(compId));
					#endif
				}

				factory->define( name, entity );
			}
		}

		return nullptr;
	}

} // namespace fggl::entity