/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 26/06/22.
//

#include <cstdlib>

#include "fggl/platform/fallback/paths.hpp"

namespace fggl::platform {

	inline static std::filesystem::path get_user_path(const char* env, const char* fallback) {
		const char* path = std::getenv(env);
		if (path != nullptr) {
			return {path};
		}
		return std::filesystem::current_path() / fallback;
	}

	EnginePaths calc_engine_paths(const char* base) {
		return EnginePaths {
			get_user_path(ENV_USER_CONFIG, DEFAULT_USER_CONFIG) / base,
			get_user_path(ENV_USER_DATA, DEFAULT_USER_DATA) / base,
			std::filesystem::temp_directory_path() / base
		};
	}

	std::filesystem::path locate_data(const EnginePaths &paths, const std::filesystem::path &relPath) {
		auto userPath = paths.userData / relPath;
		if ( std::filesystem::exists(userPath) ) {
			return userPath;
		}

		// if debug mode, try CWD as well.
		auto debugPath = std::filesystem::current_path() / "data" / relPath;
		if ( std::filesystem::exists(debugPath) ) {
			return debugPath;
		}

		// if the file existed, it should exist in the user space
		return userPath;
	}

	std::filesystem::path locate_config(const EnginePaths &paths, const std::filesystem::path &relPath) {
		return paths.userConfig / relPath;
	}

	std::filesystem::path locate_cache(const EnginePaths &paths, const std::filesystem::path &relPath) {
		return paths.userCache / relPath;
	}

} // namespace fggl::platform
