/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_DEBUG_DEBUG_H
#define FGGL_DEBUG_DEBUG_H

#include <string>
#include <memory>
#include <utility>
#include <functional>
#include <unordered_map>

#include "fggl/display/glfw/window.hpp"

namespace fggl::debug {

	using DebugUIDraw = std::function<void(bool *)>;

	struct DebugWindow {
		bool m_visible;
		DebugUIDraw m_callback;
	};

	class DebugUI {
		public:
			explicit DebugUI(std::shared_ptr<display::glfw::Window> &window);
			~DebugUI();

			void frameStart();
			void draw();

			inline void addWindow(const std::string &name, DebugUIDraw window) {
				m_windows[name] = DebugWindow{true, std::move(window) };
			}

			inline void visible(bool state) {
				m_visible = state;
			}

			inline bool visible() const {
				return m_visible;
			}

		private:
			bool m_visible;
			std::unordered_map<std::string, DebugWindow> m_windows;
	};

}

#endif
